@@ -10,6 +10,7 @@ use DateTime;
 use Digest::SHA1;
 use File::Find::Rule;
 use Git::PurePerl::Actor;
+use Git::PurePerl::Config;
 use Git::PurePerl::DirectoryEntry;
 use Git::PurePerl::Loose;
 use Git::PurePerl::Object;
@@ -34,7 +35,10 @@ use Git::PurePerl::Protocol;
 use IO::Digest;
 use IO::Socket::INET;
 use Path::Class;
-our $VERSION = '0.42_01';
+use namespace::autoclean;
+
+our $VERSION = '0.46_01';
+$VERSION = eval $VERSION;
 
 has 'directory' => (
     is       => 'ro',
@@ -75,6 +79,16 @@ has 'description' => (
     }
 );
 
+has 'config' => (
+    is      => 'ro',
+    isa     => 'Git::PurePerl::Config',
+    lazy    => 1,
+    default => sub {
+        my $self = shift;
+        Git::PurePerl::Config->new(git => $self);
+    }
+);
+
 __PACKAGE__->meta->make_immutable;
 
 sub BUILDARGS {
@@ -162,13 +176,24 @@ sub ref_sha1 {
     my ( $self, $wantref ) = @_;
     my @refs;
     my $dir = dir( $self->gitdir, 'refs' );
-    next unless -d $dir;
+    return unless -d $dir;
+
+    if ($wantref eq "HEAD") {
+        my $file = file($self->gitdir, 'HEAD');
+        my $sha1 = file($file)->slurp
+            || confess("Error reading $file: $!");
+        chomp $sha1;
+        return $self->ref_sha1($1) if $sha1 =~ /^ref: (.*)/;
+        return $sha1;
+    }
+
     foreach my $file ( File::Find::Rule->new->file->in($dir) ) {
         my $ref = 'refs/' . file($file)->relative($dir)->as_foreign('Unix');
         if ( $ref eq $wantref ) {
             my $sha1 = file($file)->slurp
                 || confess("Error reading $file: $!");
             chomp $sha1;
+            return $self->ref_sha1($1) if $sha1 =~ /^ref: (.*)/;
             return $sha1;
         }
     }
@@ -179,6 +204,7 @@ sub ref_sha1 {
             next if $line =~ /^#/;
             my ( $sha1, my $name ) = split ' ', $line;
             if ( $name eq $wantref ) {
+                return $self->ref_sha1($1) if $sha1 =~ /^ref: (.*)/;
                 return $sha1;
             }
         }
@@ -201,6 +227,16 @@ sub master {
     return $self->ref('refs/heads/master');
 }
 
+sub head_sha1 {
+    my $self = shift;
+    return $self->ref_sha1('HEAD');
+}
+
+sub head {
+    my $self = shift;
+    return $self->ref('HEAD');
+}
+
 sub get_object {
     my ( $self, $sha1 ) = @_;
     return unless $sha1;
@@ -381,6 +417,7 @@ sub init {
 
 sub checkout {
     my ( $self, $directory, $tree ) = @_;
+    $directory ||= $self->directory;
     $tree ||= $self->master->tree;
     confess("Missing tree") unless $tree;
     foreach my $directory_entry ( $tree->directory_entries ) {
@@ -402,10 +439,20 @@ sub checkout {
 }
 
 sub clone {
-    my ( $self, $hostname, $project ) = @_;
+    my $self = shift;
+
+    my $remote;
+    if (@_  == 2) {
+        # For backwards compatibility
+        $remote = "git://$_[0]";
+        $remote .= "/" unless $_[1] =~ m{^/};
+        $remote .= $_[1];
+    } else {
+        $remote = shift;
+    }
+
     my $protocol = Git::PurePerl::Protocol->new(
-        hostname => $hostname,
-        project  => $project,
+        remote => $remote,
     );
 
     my $sha1s = $protocol->connect;
@@ -426,6 +473,7 @@ sub clone {
 sub _add_file {
     my ( $class, $filename, $contents ) = @_;
     my $fh = $filename->openw || confess "Error opening to $filename: $!";
+    binmode($fh); #important for Win32
     $fh->print($contents) || confess "Error writing to $filename: $!";
     $fh->close || confess "Error closing $filename: $!";
 }
@@ -484,9 +532,19 @@ Leon Brocard <acme@astray.com>
 
 =head1 CONTRIBUTORS
 
-    Chris Reinhardt
-    Tomas (t0m) Doran
-    Dan (broquaint) Brook
+=over 4
+
+=item Chris Reinhardt
+
+=item Tomas (t0m) Doran
+
+=item Dan (broquaint) Brook
+
+=item Alex Vandiver
+
+=item Dagfinn Ilmari MannsE<aring>ker
+
+=back
 
 =head1 COPYRIGHT
 
